QT += core gui widgets network
TEMPLATE = app

TARGET = "parallel-launcher"

CONFIG += qt
CONFIG(debug, debug|release){
	OBJECTS_DIR = "build/debug/obj"
	MOC_DIR = "build/debug/moc"
	UI_DIR = "build/debug/ui"
	DEFINES += DEBUG
	!win32 {
		QMAKE_LFLAGS += -rdynamic
	}
}
CONFIG(release, debug|release){
	OBJECTS_DIR = "build/release/obj"
	MOC_DIR = "build/release/moc"
	UI_DIR = "build/release/ui"
}

DEFINES += QT_DEPRECATED_WARNINGS

SOURCES += src/main.cpp src/core/*.cpp src/db/*.cpp src/ui/*.cpp src/polyfill/*.cpp src/input/*.cpp src/plugins/*.cpp src/thirdparty/*.cpp src/thirdparty/*.c src/rhdc/core/*.cpp src/rhdc/web/*.cpp src/rhdc/ui/*.cpp
HEADERS += src/types.hpp src/core/*.hpp src/db/*.hpp src/ui/*.hpp src/polyfill/*.hpp src/input/*.hpp src/plugins/*.hpp src/thirdparty/*.h src/rhdc/core/*.hpp src/rhdc/web/*.hpp src/rhdc/ui/*.hpp
FORMS += src/ui/designer/*.ui src/rhdc/ui/designer/*.ui
RESOURCES = data/resources.qrc
TRANSLATIONS = lang/parallel-launcher.en_us.ts lang/parallel-launcher.it.ts lang/parallel-launcher.es.ts lang/parallel-launcher.eo.ts lang/parallel-launcher.fr.ts lang/parallel-launcher.fr_ca.ts lang/parallel-launcher.de.ts

macx|win32 {
	SOURCES += src/windows-mac-updater/*.cpp
	HEADERS += src/windows-mac-updater/*.hpp
	FORMS += src/windows-mac-updater/*.ui
	LIBS += -ldiscord-rpc
	VERSION = 5.2.0
}

macx {
	CONFIG += c++17
	DEFINES += BOOST_FILESYSTEM_DYN_LINK _LIBCPP_DISABLE_AVAILABILITY
	LIBS += -L./lib
	ICON = appicon.icns
	QMAKE_INFO_PLIST = app.plist
}

win32 {
	win32:contains(QMAKE_HOST.arch, x86_64) {
		WIN_DIR = win64
	} else {
		WIN_DIR = win32
	}
	
	CONFIG(debug, debug|release){
		WIN_BUILD_DIR = debug
	}
	CONFIG(release, debug|release){
		WIN_BUILD_DIR = release
	}
	
	DEFINES += _SILENCE_CXX17_CODECVT_HEADER_DEPRECATION_WARNING
	SOURCES += src/polyfill/windows/*.cpp
	HEADERS += src/polyfill/windows/*.hpp

	CONFIG += embed_manifest_exe
	QMAKE_CXXFLAGS += /std:c++17 /WX /utf-8 /Zc:preprocessor /Wv:18
	LIBS += -L$${WIN_DIR}/lib -lkernel32 -lOle32 -lSDL2 -lAdvapi32 -lsqlite3
	INCLUDEPATH += $${WIN_DIR}/include
	
	RC_ICONS = "data\\appicon.ico"
	RC_LANG = "EN-CA"
	QMAKE_TARGET_COMPANY = "Matt Pharoah"
	QMAKE_TARGET_COPYRIGHT = "GNU GENERAL PUBLIC LICENSE Version 3"
	QMAKE_TARGET_PRODUCT = "Parallel Launcher"
	
	lsjsTarget.target = parallel-launcher-lsjs.exe
	lsjsTarget.depends = FORCE
	lsjsTarget.commands = cl /WX /Wv:18 /O2 /I $${WIN_DIR}\\include lsjs.c /link $${WIN_DIR}\\lib\\SDL2.lib /out:$${WIN_BUILD_DIR}\\parallel-launcher-lsjs.exe
	
	lsjsStub.target = data/parallel-launcher-lsjs
	lsjsStub.depends = FORCE
	lsjsStub.commands = type nul > data\\parallel-launcher-lsjs
	
	PRE_TARGETDEPS += parallel-launcher-lsjs.exe data/parallel-launcher-lsjs
	QMAKE_EXTRA_TARGETS += lsjsTarget lsjsStub
}

unix {
	!macx {
		SOURCES += src/flatpak/*.cpp
		HEADERS += src/flatpak/*.hpp
		FORMS += src/flatpak/*.ui
		PKGCONFIG += flatpak
		DEFINES += DISCORD_LAZY_LINKING
		LIBS += -ldl
	}
}

!win32 {
	CONFIG += object_parallel_to_source
	QMAKE_CXXFLAGS += -std=c++17
	LIBS += -lsqlite3 -lgcrypt -lpthread
	
	macx {
		LIBS += -lboost_system -lboost_filesystem -framework CoreFoundation -framework IOKit
		INCLUDEPATH += /usr/local/include
	} else {
		CONFIG += link_pkgconfig
		QMAKE_LFLAGS += -no-pie -fno-pie
		QMAKE_CXXFLAGS += -Werror
		LIBS += -lstdc++fs -lSDL2
		
		lsjsTarget.target = data/parallel-launcher-lsjs
		lsjsTarget.depends = FORCE
		lsjsTarget.commands = gcc -std=c99 -O3 lsjs.c -o data/parallel-launcher-lsjs -lSDL2
		PRE_TARGETDEPS += data/parallel-launcher-lsjs
		QMAKE_EXTRA_TARGETS += lsjsTarget
	}
}
