#ifndef SRC_UI_MAIN_WINDOW_HPP_
#define SRC_UI_MAIN_WINDOW_HPP_

#include <QWidget>
#include <QMainWindow>
#include <memory>
#include <vector>
#include "src/core/controller.hpp"
#include "src/core/settings.hpp"
#include "src/core/async.hpp"
#include "src/core/rom.hpp"

namespace Ui {
	class MainWindow;
}

class MainWindow : public QMainWindow {
	Q_OBJECT

	private:
	Ui::MainWindow *m_ui;
	bool m_showAllPlugins;
	AppSettings m_settings;
	std::shared_ptr<bool> m_exists;
	CancellationToken m_romSearch;

	void play( bool multiplayer );
	void updateRhdcActions();
	RomInfo *getSelectedRomInfo();
	void refetchAll();

	private slots:
	void reloadSettings();
	void refreshRomList();

	void romSelectionChanged();

	void editSave( fs::path saveFilePath );
	void playSingleplayer();
	void playMultiplayer();
	void configureController();
	void configureKeyboard();
	void manageRomSources();
	void editSettings();
	void download();

	void inputModeChanged();
	void emulatorChanged();
	void parallelPluginChanged();
	void mupenPluginChanged();
	void overclockCpuChanged( bool );
	void overclockViChanged( bool );
	void widescreenChanged( bool );
	void upscaleTexRectsChanged( bool );
	void emulateFramebufferChanged( bool );
	void accurateDepthCompareChanged( bool );
	void showMorePluginsChanged( bool );
	void rhdcViewToggled();

	void rhdcLogin();
	void rhdcLogout();
	void rhdcDisable();

	public:
	MainWindow();
	virtual ~MainWindow();

	protected:
	virtual void closeEvent( QCloseEvent *event ) override;
	virtual void keyPressEvent( QKeyEvent *event ) override;

};



#endif /* SRC_UI_MAIN_WINDOW_HPP_ */
